@@ -1,4 +1,4 @@
-__version__ = "0.0.3"
+__version__ = "0.0.4"
 
 # optimizers first
 # rest
@@ -43,4 +43,6 @@ __all__ = [
     "Map",
     "Apply",
     "Extensions",
+    "Compact",
+    "compact",
 ]
